Documentation
¶
Overview ¶
Package skills contains the embedded skill definitions for tfctl.
Index ¶
Constants ¶
const TFCTLSkillPath = "tfctl/SKILL.md"
TFCTLSkillPath is the path to the embedded SKILL.md file within the binary.
Variables ¶
var AgentNames []string
AgentNames is a list of the names of all supported agents.
var FS embed.FS
FS is the embedded filesystem containing the skill definitions for tfctl.
Functions ¶
This section is empty.
Types ¶
type AgentSpec ¶
type AgentSpec struct {
Name string
DisplayName string
SkillsDir string
GlobalSkillsDir func() string
Detect func() bool
DetectParentProcess func() bool
}
AgentSpec defines the necessary information to install a skill for a coding agent.
func DetectAgent ¶
func DetectAgent() []AgentSpec
DetectAgent returns a list of AgentSpecs for agents detected on the current system.
func GetAgent ¶
GetAgent returns the AgentSpec for a given agent name, along with a boolean indicating whether the agent was found.
func (AgentSpec) InstallSkill ¶
InstallSkill installs the tfctl skill for the agent, either to the project directory or the global config directory based on the value of the global parameter.